@@ -12,6 +12,27 @@ pub struct Square {
 }
 
 impl Square {
+    pub fn new(column: u8, row: u8) -> Self {
+        assert!(column < 8);
+        assert!(row < 8);
+        Self { column, row }
+    }
+
+    pub fn shift(self, direction: Direction, amount: i8) -> Self {
+        let Self { column, row } = self;
+        let (x, y) = direction.offset();
+        Self::new(
+            x.checked_mul(amount)
+                .unwrap()
+                .checked_add(column as i8)
+                .unwrap() as u8,
+            y.checked_mul(amount)
+                .unwrap()
+                .checked_add(row as i8)
+                .unwrap() as u8,
+        )
+    }
+
     pub fn column(self) -> u8 {
         self.column
     }
@@ -19,6 +40,46 @@ impl Square {
     pub fn row(self) -> u8 {
         self.row
     }
+
+    const fn assert_on_board(self, board_size: u8) {
+        assert!(self.row < board_size);
+        assert!(self.column < board_size);
+    }
+
+    /// Generate a Square that is one step in the direction from this Square.
+    /// If the generated Square is outside of the board, then this returns None instead.
+    pub fn checked_step(self, direction: Direction, board_size: u8) -> Option<Self> {
+        self.assert_on_board(board_size);
+        use Direction::*;
+        let (column, row) = (self.column, self.row);
+        let (column, row) = match direction {
+            Up => (column, row.checked_add(1).filter(|&y| y < board_size)?),
+            Down => (column, row.checked_sub(1)?),
+            Left => (column.checked_sub(1)?, row),
+            Right => (column.checked_add(1).filter(|&x| x < board_size)?, row),
+        };
+        Some(Self { column, row })
+    }
+
+    /// Rotate 1 quarter turn clockwise.
+    #[must_use]
+    pub const fn rotate(self, board_size: u8) -> Self {
+        self.assert_on_board(board_size);
+        Self {
+            column: self.row,
+            row: board_size - 1 - self.column,
+        }
+    }
+
+    /// Mirror along the horizontal axis.
+    #[must_use]
+    pub const fn mirror(self, board_size: u8) -> Self {
+        self.assert_on_board(board_size);
+        Self {
+            column: self.column,
+            row: board_size - 1 - self.row,
+        }
+    }
 }
 
 impl Display for Square {
@@ -101,6 +162,30 @@ impl Direction {
             Self::Left => (-1, 0),
         }
     }
+
+    /// Rotate 1 quarter turn clockwise.
+    #[must_use]
+    pub const fn rotate(self) -> Self {
+        use Direction::*;
+        match self {
+            Up => Right,
+            Down => Left,
+            Left => Up,
+            Right => Down,
+        }
+    }
+
+    /// Mirror along the horizontal axis. Up and Down get flipped.
+    #[must_use]
+    pub const fn mirror(self) -> Self {
+        use Direction::*;
+        match self {
+            Up => Down,
+            Down => Up,
+            Left => Left,
+            Right => Right,
+        }
+    }
 }
 
 impl From<Direction> for (i8, i8) {
@@ -191,6 +276,11 @@ impl Iterator for DropCounts {
 pub struct Pattern(u8);
 
 impl Pattern {
+    pub fn from_mask(mask: u8) -> Self {
+        assert!(mask != 0x00 && mask != 0xFF);
+        Self(mask)
+    }
+
     pub fn mask(self) -> u8 {
         self.0
     }
@@ -216,6 +306,16 @@ impl Pattern {
     }
 }
 
+impl FromIterator<u32> for Pattern {
+    fn from_iter<T: IntoIterator<Item = u32>>(iter: T) -> Self {
+        Self::from_mask(iter.into_iter().fold(0u8, |acc, count| {
+            let shift = count - 1;
+            assert!(shift < acc.trailing_zeros());
+            ((acc >> 1) | 0x80) >> shift
+        }))
+    }
+}
+
 impl Display for Pattern {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         if self.count_squares() == 1 {
@@ -432,7 +532,7 @@ impl FromStr for Move {
 
         Ok(Self {
             square,
-            kind: if rest.len() == 0 {
+            kind: if rest.is_empty() {
                 if taken_count != None {
                     Err(TruncatedSpread)?
                 } else {
@@ -528,4 +628,66 @@ mod tests {
         error::<Move, _, _>(["2c1+111"], CountMismatch);
         error::<Move, _, _>(["3d5<*"], BadCrush);
     }
+
+    #[test]
+    fn checked_step() {
+        let s = Square::new;
+        assert_eq!(s(2, 1).checked_step(Direction::Up, 3), Some(s(2, 2)));
+        assert_eq!(s(2, 3).checked_step(Direction::Up, 4), None);
+        assert_eq!(s(4, 5).checked_step(Direction::Down, 6), Some(s(4, 4)));
+        assert_eq!(s(4, 0).checked_step(Direction::Down, 6), None);
+        assert_eq!(s(2, 1).checked_step(Direction::Left, 3), Some(s(1, 1)));
+        assert_eq!(s(0, 1).checked_step(Direction::Left, 3), None);
+        assert_eq!(s(0, 0).checked_step(Direction::Right, 2), Some(s(1, 0)));
+        assert_eq!(s(0, 0).checked_step(Direction::Right, 1), None);
+    }
+
+    #[test]
+    fn rotate_even() {
+        let s = Square::new;
+        // corner
+        let sq = s(0, 0);
+        assert_eq!(sq.rotate(6), s(0, 5));
+        assert_eq!(sq.rotate(6).rotate(6), s(5, 5));
+        assert_eq!(sq.rotate(6).rotate(6).rotate(6), s(5, 0));
+        assert_eq!(sq.rotate(6).rotate(6).rotate(6).rotate(6), s(0, 0));
+        // centre
+        let sq = s(2, 2);
+        assert_eq!(sq.rotate(6), s(2, 3));
+        assert_eq!(sq.rotate(6).rotate(6), s(3, 3));
+        assert_eq!(sq.rotate(6).rotate(6).rotate(6), s(3, 2));
+        assert_eq!(sq.rotate(6).rotate(6).rotate(6).rotate(6), s(2, 2));
+    }
+
+    #[test]
+    fn rotate_odd() {
+        let s = Square::new;
+        // corner
+        let sq = s(0, 0);
+        assert_eq!(sq.rotate(7), s(0, 6));
+        assert_eq!(sq.rotate(7).rotate(7), s(6, 6));
+        assert_eq!(sq.rotate(7).rotate(7).rotate(7), s(6, 0));
+        assert_eq!(sq.rotate(7).rotate(7).rotate(7).rotate(7), s(0, 0));
+        // centre
+        let sq = s(3, 3);
+        assert_eq!(sq.rotate(7), s(3, 3));
+    }
+
+    #[test]
+    fn mirror_even() {
+        let square = Square::new(1, 2);
+        assert_eq!(square.mirror(6), Square::new(1, 3));
+        assert_eq!(square.mirror(6).mirror(6), Square::new(1, 2));
+    }
+
+    #[test]
+    fn mirror_odd() {
+        let square = Square::new(4, 1);
+        assert_eq!(square.mirror(7), Square::new(4, 5));
+        assert_eq!(square.mirror(7).mirror(7), Square::new(4, 1));
+
+        // centre line
+        let square = Square::new(2, 3);
+        assert_eq!(square.mirror(7), Square::new(2, 3));
+    }
 }
